@@ -123,13 +123,17 @@ NULL
 #' @include ExperimentList-class.R
 setClass(
     "MultiAssayExperiment",
-     slots = list(
-       ExperimentList = "ExperimentList",
-       colData = "DataFrame",
-       sampleMap = "DataFrame",
-       drops = "list"
-     ),
-    contains = "Annotated"
+    contains = "Annotated",
+    slots = list(
+        ExperimentList = "ExperimentList",
+        colData = "DataFrame",
+        sampleMap = "DataFrame",
+        drops = "list"
+    ),
+    prototype = prototype(
+        colData = new("DFrame"),
+        sampleMap = new("DFrame")
+    )
 )
 
 ### ==============================================
@@ -788,23 +792,35 @@ setMethod("updateObject", "MultiAssayExperiment",
     function(object, ..., verbose = FALSE) {
         if (verbose)
             message("updateObject(object = 'MultiAssayExperiment')")
-        oldAPI <- try(object@ExperimentList, silent = TRUE)
-        new(class(object),
-            ExperimentList = if (is(oldAPI, "try-error"))
-                ExperimentList(object@Elist@listData)
-            else experiments(object),
-            colData = if (is(try(object@colData, silent = TRUE),
-                             "try-error"))
-                object@pData
-            else
-                colData(object),
-            sampleMap = if (is(oldAPI, "try-error"))
-                .rearrangeMap(sampleMap(object))
-            else sampleMap(object),
-            metadata = metadata(object),
-            drops = getElement(object, "drops")
+
+        oldEL <- try(object@ExperimentList, silent = TRUE)
+        if (is(oldEL, "try-error")) {
+            explist <- ExperimentList(object@Elist@listData)
+            samplemap <- .rearrangeMap(object@sampleMap)
+        } else {
+            explist <- experiments(object)
+            samplemap <- sampleMap(object)
+        }
+
+        oldCD <- try(object@colData, silent = TRUE)
+        if (is(oldCD, "try-error"))
+            coldata <- object@pData
+        else
+            coldata <- colData(object)
+
+        explist <- updateObject(explist, ..., verbose = verbose)
+        coldata <- updateObject(coldata, ..., verbose = verbose)
+        samplemap <- updateObject(samplemap, ..., verbose = verbose)
+
+        BiocGenerics:::replaceSlots(
+            object,
+            ExperimentList = explist,
+            colData = coldata,
+            sampleMap = samplemap,
+            check=FALSE
         )
-    })
+    }
+)
 
 .mergeColData <- function(inlist) {
     CDbyEXP <- lapply(names(inlist),
