@@ -39,9 +39,14 @@ NULL
 #'     that have a \link[=RangedSummarizedExperiment-class]{rowRanges} method
 #'     \item getWithColData: A convenience function for extracting an assay
 #'     and associated colData
+#'     \item renamePrimary: A convenience function to rename the primary
+#'     biological units as represented in the \code{rownames(colData)}
+#'     \item renameColname: A convenience function to rename the colnames
+#'     of a particular assay
 #' }
 #'
 #' @param x A MultiAssayExperiment or ExperimentList
+#'
 #' @param ... Additional arguments. See details for more information.
 #'
 #' @return See the itemized list in the description section for details.
@@ -227,7 +232,8 @@ setMethod("mergeReplicates", "ExperimentList",
                     replicates = replicate[[i]], ...)
             }, element = x, simply = simplify, replicate = replicates, ...)
         ExperimentList(redList)
-    })
+    }
+)
 
 #' @rdname MultiAssayExperiment-helpers
 #'
@@ -345,24 +351,26 @@ setMethod("mergeReplicates", "ANY",
 #' to provide a consistent long and skinny \link{DataFrame}.
 #'
 #' @section longFormat:
-#' The longFormat method takes data from the \code{\link{ExperimentList}}
+#' The 'longFormat' method takes data from the \code{\link{ExperimentList}}
 #' in a \code{\link{MultiAssayExperiment}} and returns a uniform
 #' \code{\link{DataFrame}}. The resulting DataFrame has columns indicating
 #' primary, rowname, colname and value. This method can optionally include
-#' colData columns with the \code{colDataCols} argument
-#' (\code{MultiAssayExperiment} method only). The \code{i} argument
-#' allows the user to specify the assay value in a
-#' \linkS4class{SummarizedExperiment}. It directly relates to the \code{i}
-#' argument in the assay method.
+#' columns of the MultiAssayExperiment colData named by \code{colDataCols} character
+#' vector argument. (\code{MultiAssayExperiment} method only). The \code{i} argument
+#' allows the user to specify the assay value for the
+#' \linkS4class{SummarizedExperiment} assay function's \code{i} argument.
 #'
 #' @param object Any supported class object
 #'
 #' @param colDataCols A \code{character}, \code{logical}, or \code{numeric}
 #'     index for \code{colData} columns to be included
 #'
-#' @param i The assay indicator for \linkS4class{SummarizedExperiment}
-#'     objects. A vector input is supported in the case that the
-#'     SummarizedExperiment object(s) has more than one assay (default 1L)
+#' @param i longFormat: The i-th assay in
+#'     \linkS4class{SummarizedExperiment}-like objects. A vector input is
+#'     supported in the case that the SummarizedExperiment object(s) has more
+#'     than one assay (default 1L),
+#'     renameColname: Either a \code{numeric} or \code{character} index
+#'     indicating the assay whose colnames are to be renamed
 #'
 #' @param mode String indicating how \linkS4class{MultiAssayExperiment}
 #'     column-level metadata should be added to the
@@ -375,6 +383,9 @@ longFormat <- function(object, colDataCols = NULL, i = 1L) {
     else if (!is(object, "MultiAssayExperiment"))
         return(.longFormatANY(object, i = i))
 
+    if (any(.emptyAssays(experiments(object))))
+        object <- .dropEmpty(object, warn = FALSE)
+
     longDataFrame <- do.call(function(...) rbind(..., make.row.names = FALSE),
         .longFormatElist(experiments(object), i = i))
 
@@ -443,12 +454,12 @@ longFormat <- function(object, colDataCols = NULL, i = 1L) {
 #' The \code{wideFormat} function returns standardized wide \link{DataFrame}
 #' where each row represents a biological unit as in the \code{colData}.
 #' Depending on the data and setup, biological units can be patients, tumors,
-#' specimens, etc. Optionally, \code{colData} columns can be added to the
-#' wide data output (see the \code{colDataCols} argument). Metadata columns are
+#' specimens, etc. Metadata columns are
 #' generated based on the names produced in the wide format
 #' \linkS4class{DataFrame}. These can be accessed via the
-#' \link[=Vector-class]{mcols} function. See the \code{Arguments} and
-#' \code{longFormat} sections for argument descriptions.
+#' \link[=Vector-class]{mcols} function.
+#' See the \code{wideFormat} section for description of the \code{colDataCols} and
+#' \code{i} arguments.
 #'
 #' @param check.names (logical default TRUE) Column names of the output
 #' \code{DataFrame} will be checked for syntactic validity and made unique,
@@ -465,6 +476,9 @@ wideFormat <- function(object, colDataCols = NULL, check.names = TRUE,
     collSymbol <- "///"
     key <- "feature"
 
+    if (any(.emptyAssays(experiments(object))))
+        object <- .dropEmpty(object, warn = FALSE)
+
     if (is.null(colDataCols)) colDataCols <- character(0L)
     nameFUN <- if (check.names) make.names else I
     cnames <- colnames(object)
@@ -509,7 +523,7 @@ wideFormat <- function(object, colDataCols = NULL, check.names = TRUE,
     }
 
     wideData <- lapply(wideData, function(flox) {
-        flox <- tidyr::spread(flox, key = {key}, value = "value")
+        flox <- tidyr::pivot_wider(flox, names_from = key)
         .matchAddColData(flox, colData(object), colDataCols)
     })
     wideDF <- Reduce(function(x, y)
@@ -562,6 +576,9 @@ setMethod("hasRowRanges", "ExperimentList", function(x) {
 
 #' @rdname MultiAssayExperiment-helpers
 #'
+#' @param verbose logical(1) Whether to `suppressMessages` on subsetting
+#'     operations in `getWithColData` (default FALSE)
+#'
 #' @aliases getWithColData
 #'
 #' @section getWithColData:
@@ -585,7 +602,7 @@ setMethod("hasRowRanges", "ExperimentList", function(x) {
 #' no replacement or appending is performed.
 #'
 #' @export getWithColData
-getWithColData <- function(x, i, mode=c("append", "replace")) {
+getWithColData <- function(x, i, mode=c("append", "replace"), verbose = FALSE) {
     if (!is(x, "MultiAssayExperiment"))
         stop("Provide a MultiAssayExperiment as input")
 
@@ -593,7 +610,8 @@ getWithColData <- function(x, i, mode=c("append", "replace")) {
         identical(length(i), 1L), !is.na(i), !is.logical(i),
         is.character(mode), !is.na(mode), !is.logical(mode))
 
-    mae <- x[, , i, drop = TRUE]
+    FUN <- if (!verbose) suppressMessages else force
+    mae <- FUN(x[, , i, drop = FALSE])
     prims <- sampleMap(mae)[["primary"]]
     if (anyDuplicated(prims))
         warning("Duplicating colData rows due to replicates in 'replicated(x)'",
@@ -632,3 +650,163 @@ getWithColData <- function(x, i, mode=c("append", "replace")) {
     exObj
 }
 
+#' @rdname MultiAssayExperiment-helpers
+#'
+#' @aliases renamePrimary
+#'
+#' @section rename*:
+#' The \code{renamePrimary} function allows the user to conveniently change the
+#' actual names of the primary biological units as seen in
+#' \code{rownames(colData)}. \code{renameColname} allows the user to change the
+#' names of a particular assay based on index \code{i}. \code{i} can either be
+#' a single numeric or character value. See \code{colnames<-} method for
+#' renaming multiple colnames in a \code{MultiAssayExperiment}.
+#'
+#' @param value renamePrimary: A \code{character} vector of the same length as
+#' the existing \code{rownames(colData)} to use for replacement,
+#' renameColname: A \code{CharacterList} or \code{list} with matching
+#' \code{lengths} to replace \code{colnames(x)}
+#'
+#' @examples
+#'
+#' ## renaming biological units (primary)
+#'
+#' mae2 <- renamePrimary(mae, paste0("pt", 1:4))
+#' colData(mae2)
+#' sampleMap(mae2)
+#'
+#' @export renamePrimary
+renamePrimary <- function(x, value) {
+    coldata <- colData(x)
+    old <- rownames(coldata)
+    if (length(old) != length(value))
+        stop("'value' length does not match 'length(rownames(colData))'")
+    samplemap <- sampleMap(x)
+    nprime <- value[match(samplemap[["primary"]], old)]
+    samplemap[["primary"]] <- nprime
+    rownames(coldata) <- value
+    BiocGenerics:::replaceSlots(
+        object = x,
+        sampleMap = samplemap,
+        colData = coldata
+    )
+}
+
+#' @rdname MultiAssayExperiment-helpers
+#'
+#' @aliases renameColname
+#'
+#' @examples
+#'
+#' ## renaming observational units (colname)
+#'
+#' mae2 <- renameColname(mae, i = "Affy", paste0("ARRAY", 1:4))
+#' colnames(mae2)
+#' sampleMap(mae2)
+#'
+#'
+#' @export renameColname
+renameColname <- function(x, i, value) {
+    stopifnot(length(i) == 1L, !is.na(i), !missing(i))
+    exps <- experiments(x)
+    expAssay <- x[[i]]
+    splitmp <- mapToList(sampleMap(x))
+    expmap <- splitmp[[i]]
+    old <- expmap[["colname"]]
+    if (length(old) != length(value))
+        stop("'value' length does not match 'length(colnames(x[[i]]))'")
+    newcns <- value[match(expmap[["colname"]], old)]
+    expmap[["colname"]] <- newcns
+    splitmp[[i]] <- expmap
+    smp <- listToMap(splitmp)
+    colnames(expAssay) <- value
+    exps[[i]] <- expAssay
+    BiocGenerics:::replaceSlots(
+        object = x,
+        sampleMap = smp,
+        ExperimentList = exps
+    )
+}
+
+#' @rdname MultiAssayExperiment-helpers
+#'
+#' @aliases splitAssays
+#'
+#' @section splitAssays:
+#' The \code{splitAssays} method separates columns in each of the assays based
+#' on the \code{hitList} input. The \code{hitList} can be generated using
+#' the \code{makeHitList} helper function. To use the \code{makeHitList}
+#' helper, the user should input a list of patterns that will match on the
+#' column names of each assay. These matches should be mutually exclusive as
+#' to avoid repetition of columns across assays. See the examples section.
+#'
+#' @param hitList a named \code{list} or \code{List} of logical vectors that
+#' indicate groupings in the assays
+#'
+#' @param patternList a named \code{list} or \code{List} of atomic character
+#' vectors that are the input to \code{grepl} for identifying groupings in
+#' the assays
+#'
+#' @export hasRowRanges
+setGeneric("splitAssays", function(x, hitList)
+        standardGeneric("splitAssays")
+)
+
+#' @rdname MultiAssayExperiment-helpers
+#' @exportMethod splitAssays
+setMethod("splitAssays", "MultiAssayExperiment",
+    function(x, hitList) {
+        stopifnot(is.list(hitList) || is(hitList, "List"))
+        exps <- experiments(x)
+        innames <- lapply(hitList, names)
+        validENames <- unique(unlist(innames))
+        exps <- exps[names(exps) %in% validENames]
+        sublist <- lapply(hitList, function(logilist) {
+            Map(function(x, y) {
+                x[, y, drop = FALSE]
+            }, x = exps, y = logilist)
+        })
+        sublist <- unlist(sublist, recursive = FALSE)
+        names(sublist) <- gsub("\\.", "_", names(sublist))
+
+        BiocGenerics:::replaceSlots(
+            object = x,
+            ExperimentList = ExperimentList(sublist)
+        )
+    }
+)
+
+#' @rdname MultiAssayExperiment-helpers
+#'
+#' @aliases makeMatchList
+#'
+#' @examples
+#'
+#' patts <- list(
+#'     normals = "TCGA-OR-[A-Z0-9]{4}-11",
+#'     tumors = "TCGA-OR-[A-Z0-9]{4}-01"
+#' )
+#'
+#' data("miniACC")
+#'
+#' hits <- makeHitList(miniACC, patts)
+#'
+#' splitAssays(miniACC, hits)
+#'
+#' @export
+makeHitList <- function(x, patternList) {
+    Colnames <- colnames(x)
+    res <- lapply(
+        stats::setNames(nm = names(patternList)),
+        function(pattname, patt) {
+            grepl(patt[[pattname]], unlist(Colnames))
+        }, patt = patternList
+    )
+    reslist <- lapply(res, relist, Colnames)
+    sums <- do.call(function(...) Map(`+`, ...), reslist)
+    if (any(unlist(sums) > 1))
+        stop("Groupings are not mutually exclusive")
+    matching <- vapply(reslist, function(x) any(any(x)), logical(1L))
+    reslist[matching]
+}
+
